require "auditing/middleware/version"
require 'soar_aspects'

module Auditing
  module Middleware
    class Error < StandardError; end

    def self.error(ruby_error)
      log_detail(ENV['RACK_ENV'], ruby_error, :error)
    end

    def self.info(ruby_error)
      log_detail(ENV['RACK_ENV'], ruby_error, :info)
    end

    def self.warn(ruby_error)
      log_detail(ENV['RACK_ENV'], ruby_error, :warn)
    end

    def self.debug(ruby_error)
      log_detail(ENV['RACK_ENV'], ruby_error, :debug)
    end

    def self.fatal(ruby_error)
      log_detail(ENV['RACK_ENV'], ruby_error, :fatal)
    end

    private
    def self.log_detail(rack_env, ruby_error, audit_level)
      if rack_env == 'development'
        what_to_log = ruby_error.respond_to?(:backtrace) ? "#{ruby_error.message}  #{ruby_error.backtrace}" : ruby_error
        log_to_location(what_to_log , audit_level)
      else
        log_to_location(ruby_error, audit_level)
      end
    rescue => exception
      Error.new("Unexpected exception: #{exception}")
    end

    def self.log_to_location(what_to_log, audit_level)
      auditor = SoarAspects::Aspects.auditing
      return STDERR.puts what_to_log if auditor.nil?
      auditor.send(audit_level, what_to_log)
    end
  end
end
